@@ -188,33 +188,67 @@ class IntArg(ArgType):
         info.varlist.add('int', 'ret')
         info.codeafter.append('    return PyInt_FromLong(ret);')
 
+class TimeTArg(ArgType):
+    def write_param(self, ptype, pname, pdflt, pnull, info):
+	if pdflt:
+	    info.varlist.add('time_t', pname + ' = ' + pdflt)
+	else:
+	    info.varlist.add('time_t', pname)
+	info.arglist.append(pname)
+        info.add_parselist('i', ['&' + pname], [pname])
+    def write_return(self, ptype, ownsreturn, info):
+        info.varlist.add('time_t', 'ret')
+        info.codeafter.append('    return PyInt_FromLong(ret);')
+
 class ULongArg(ArgType):
-    dflt = ('    if (py_%(name)s && PyLong_Check(py_%(name)s))\n'
-            '        %(name)s = PyLong_AsUnsignedLong(py_%(name)s);\n'
-            '    else if (py_%(name)s) {\n'
-            '        PyErr_SetString(PyExc_TypeError, "%(name)s must be an int");\n'
-            '        return NULL;\n'
-            '    }\n')
-    before = ('    if (py_%(name)s && PyLong_Check(py_%(name)s))\n'
-            '        %(name)s = PyLong_AsUnsignedLong(py_%(name)s);\n'
-            '    else {\n'
-            '        PyErr_SetString(PyExc_TypeError, "%(name)s must be an int");\n'
-            '        return NULL;\n'
-            '    }\n')
+    dflt = '    if (py_%(name)s)\n' \
+           '        %(name)s = PyLong_AsUnsignedLong(py_%(name)s);\n'
+    before = '    %(name)s = PyLong_AsUnsignedLong(py_%(name)s);\n'
     def write_param(self, ptype, pname, pdflt, pnull, info):
         if pdflt:
-            info.varlist.add('unsigned int', pname + ' = ' + pdflt)
+            info.varlist.add('gulong', pname + ' = ' + pdflt)
             info.codebefore.append(self.dflt % {'name':pname})            
         else:
-            info.varlist.add('unsigned int', pname)
+            info.varlist.add('gulong', pname)
             info.codebefore.append(self.before % {'name':pname})            
         info.varlist.add('PyObject', "*py_" + pname + ' = NULL')
         info.arglist.append(pname)
-        info.add_parselist('O', ['&py_' + pname], [pname])
+        info.add_parselist('O!', ['&PyLong_Type', '&py_' + pname], [pname])
     def write_return(self, ptype, ownsreturn, info):
-        info.varlist.add('unsigned int', 'ret')
+        info.varlist.add('gulong', 'ret')
         info.codeafter.append('    return PyLong_FromUnsignedLong(ret);')
 
+class Int64Arg(ArgType):
+    def write_param(self, ptype, pname, pdflt, pnull, info):
+	if pdflt:
+	    info.varlist.add('gint64', pname + ' = ' + pdflt)
+	else:
+	    info.varlist.add('gint64', pname)
+	info.arglist.append(pname)
+        info.add_parselist('L', ['&' + pname], [pname])
+    def write_return(self, ptype, ownsreturn, info):
+        info.varlist.add('gint64', 'ret')
+        info.codeafter.append('    return PyLong_FromLongLong(ret);')
+
+class UInt64Arg(ArgType):
+    dflt = '    if (py_%(name)s)\n' \
+           '        %(name)s = PyLong_AsUnsignedLongLong(py_%(name)s);\n'
+    before = '    %(name)s = PyLong_AsUnsignedLongLong(py_%(name)s);\n'
+    def write_param(self, ptype, pname, pdflt, pnull, info):
+        if pdflt:
+            info.varlist.add('guint64', pname + ' = ' + pdflt)
+            info.codebefore.append(self.dflt % {'name':pname})            
+        else:
+            info.varlist.add('guint64', pname)
+            info.codebefore.append(self.before % {'name':pname})            
+        info.varlist.add('PyObject', "*py_" + pname + ' = NULL')
+        info.arglist.append(pname)
+        info.add_parselist('O!', ['&PyLong_Type', '&py_' + pname], [pname])
+    def write_return(self, ptype, ownsreturn, info):
+        info.varlist.add('guint64', 'ret')
+        info.codeafter.append('    return PyLong_FromUnsignedLongLong(ret);')
+        
+
 class DoubleArg(ArgType):
     def write_param(self, ptype, pname, pdflt, pnull, info):
 	if pdflt:
@@ -333,16 +367,8 @@ class ObjectArg(ArgType):
             '        PyErr_SetString(PyExc_TypeError, "%(name)s should be a %(type)s or None");\n'
             '        return NULL;\n'
             '    }\n')
-    dflt = ('    if (py_%(name)s && pygobject_check(py_%(name)s, &Py%(type)s_Type))\n'
-            '        %(name)s = %(cast)s(py_%(name)s->obj);\n'
-            '    else if (py_%(name)s) {\n'
-            '        PyErr_SetString(PyExc_TypeError, "%(name)s should be a %(type)s");\n'
-            '        return NULL;\n'
-            '    }\n')
-    check = ('    if (!pygobject_check(%(name)s, &Py%(type)s_Type)) {\n'
-             '        PyErr_SetString(PyExc_TypeError, "%(name)s should be a %(type)s");\n'
-             '        return NULL;\n'
-             '    }\n')
+    dflt = '    if (py_%(name)s)\n' \
+           '        %(name)s = %(cast)s(py_%(name)s->obj);\n'
     def __init__(self, objname, parent, typecode):
 	self.objname = objname
 	self.cast = string.replace(typecode, '_TYPE_', '_', 1)
@@ -368,23 +394,21 @@ class ObjectArg(ArgType):
 		info.varlist.add(self.objname, '*' + pname + ' = ' + pdflt)
 		info.varlist.add('PyGObject', '*py_' + pname + ' = NULL')
 		info.codebefore.append(self.dflt % {'name':pname,
-                                                    'cast':self.cast,
-                                                    'type':self.objname}) 
+                                                    'cast':self.cast}) 
 		info.arglist.append(pname)
-                info.add_parselist('O', ['&py_' + pname], [pname])
+                info.add_parselist('O', ['&Py%s_Type' % self.objname,
+                                         '&py_' + pname], [pname])
 	    else:
 		info.varlist.add('PyGObject', '*' + pname)
-		info.codebefore.append(self.check % {'name':pname,
-                                                     'cast':self.cast,
-                                                     'type':self.objname}) 
 		info.arglist.append('%s(%s->obj)' % (self.cast, pname))
-                info.add_parselist('O', ['&' + pname], [pname])
+                info.add_parselist('O!', ['&Py%s_Type' % self.objname,
+                                          '&' + pname], [pname])
     def write_return(self, ptype, ownsreturn, info):
         if ptype[-1] == '*': ptype = ptype[:-1]
         info.varlist.add(ptype, '*ret')
         if ownsreturn:
             info.varlist.add('PyObject', '*py_ret')
-            info.codeafter.append('    py_ret = pygobject_new((GObject *)ret;\n'
+            info.codeafter.append('    py_ret = pygobject_new((GObject *)ret);\n'
                                   '    g_object_unref(ret);\n'
                                   '    return py_ret;')
         else:
@@ -421,7 +445,15 @@ class BoxedArg(ArgType):
 	    info.codebefore.append(self.check % {'name':  pname,
                                                  'typename': self.typename,
                                                  'typecode': self.typecode})
-        info.arglist.append(pname)
+        if ptype[-1] == '*':
+            typename = ptype[:-1]
+            if typename[:6] == 'const-': typename = typename[6:]
+            if typename != self.typename:
+                info.arglist.append('(%s *)%s' % (ptype[:-1], pname))
+            else:
+                info.arglist.append(pname)
+        else:
+            info.arglist.append(pname)
         info.add_parselist('O', ['&py_' + pname], [pname])
     ret_tmpl = '    /* pyg_boxed_new handles NULL checking */\n' \
                '    return pyg_boxed_new(%(typecode)s, %(ret)s, %(copy)s, TRUE);'
@@ -432,10 +464,11 @@ class BoxedArg(ArgType):
         else:
             info.varlist.add(self.typename, 'ret')
             ret = '&ret'
+            ownsreturn = 0 # of course it can't own a ref to a local var ...
         info.codeafter.append(self.ret_tmpl %
                               { 'typecode': self.typecode,
                                 'ret': ret,
-                                'copy': ownsreturn and 'TRUE' or 'FALSE'})
+                                'copy': ownsreturn and 'FALSE' or 'TRUE'})
 
 class CustomBoxedArg(ArgType):
     # haven't done support for default args.  Is it needed?
@@ -551,7 +584,18 @@ class GErrorArg(ArgType):
 
 class GtkTreePathArg(ArgType):
     # haven't done support for default args.  Is it needed?
-    normal = '    %(name)s = pygtk_tree_path_from_pyobject(py_%(name)s);\n'
+    normal = ('    %(name)s = pygtk_tree_path_from_pyobject(py_%(name)s);\n'
+              '    if (!%(name)s) {\n'
+              '        PyErr_SetString(PyExc_TypeError, "could not convert %(name)s to a GtkTreePath");\n'
+              '        return NULL;\n'
+              '    }\n')
+    null = ('    if (py_%(name)s != Py_None) {\n'
+            '        %(name)s = pygtk_tree_path_from_pyobject(py_%(name)s);\n'
+            '        if (!%(name)s) {\n'
+            '            PyErr_SetString(PyExc_TypeError, "could not convert %(name)s to a GtkTreePath");\n'
+            '            return NULL;\n'
+            '        }\n'
+            '    }\n')
     null = ('    if (PyTuple_Check(py_%(name)s))\n'
             '        %(name)s = pygtk_tree_path_from_pyobject(py_%(name)s);\n'
             '    else if (py_%(name)s != Py_None) {\n'
@@ -570,11 +614,11 @@ class GtkTreePathArg(ArgType):
 	    info.arglist.append(pname)
             info.add_parselist('O', ['&py_' + pname], [pname])
 	else:
-            info.varlist.add('GtkTreePath', '*' + pname + ' = NULL')
+            info.varlist.add('GtkTreePath', '*' + pname)
 	    info.varlist.add('PyObject', '*py_' + pname)
             info.codebefore.append(self.normal % {'name': pname})
 	    info.arglist.append(pname)
-            info.add_parselist('O!', ['&PyTuple_Type', '&py_' + pname],[pname])
+            info.add_parselist('O', ['&py_' + pname], [pname])
         info.codeafter.append(self.freepath % {'name': pname})
     def write_return(self, ptype, ownsreturn, info):
         info.varlist.add('GtkTreePath', '*ret')
@@ -595,30 +639,19 @@ class GtkTreePathArg(ArgType):
                                   '    return Py_None;')
 						       
 class GdkRectanglePtrArg(ArgType):
-    null = ('    if (PyTuple_Check(py_%(name)s)) {\n'
-            '        %(name)s = g_new(GdkRectangle, 1);\n'
-	    '        if (!PyArg_ParseTuple(py_%(name)s, "iiii", &%(name)s->x, &%(name)s->y, &%(name)s->width, &%(name)s->height)) {\n'
-	    '            return NULL;\n'
-	    '        }\n'
-	    '    } else {\n'
-	    '        PyErr_SetString(PyExc_TypeError, "%(name)s must be a tuple");\n'
-	    '    }\n')
+    null = ('    if (!pygdk_rectangle_from_pyobject(py_%(name)s, &%(name)s))\n'
+            '        return NULL;\n')
     def write_param(self, ptype, pname, pdflt, pnull, info):
-	if pnull:
-	    info.varlist.add('GdkRectangle', '*' + pname + ' = NULL')
-	    info.varlist.add('PyObject', '*py_' + pname + ' = Py_None')
-            info.add_parselist('O', ['&py_' + pname], [pname])
-	else:
-	    info.varlist.add('GdkRectangle', '*' + pname + ' = NULL')
-	    info.varlist.add('PyObject', '*py_' + pname)
-            info.add_parselist('O!', ['&PyTuple_Type', '&py_' + pname],[pname])
-	info.arglist.append(pname)
+        info.varlist.add('GdkRectangle', pname + ' = { 0, 0, 0, 0 }')
+        info.varlist.add('PyObject', '*py_' + pname + ' = Py_None')
+        info.add_parselist('O', ['&py_' + pname], [pname])
+	info.arglist.append('&' + pname)
 	info.codebefore.append(self.null % {'name':  pname})
 
 class GdkRectangleArg(ArgType):
     def write_return(self, ptype, ownsreturn, info):
 	info.varlist.add('GdkRectangle', 'ret')
-	info.codeafter.append('    return pyg_boxed_new(GDK_RECTANGLE, ret, TRUE, TRUE);')
+	info.codeafter.append('    return pyg_boxed_new(GDK_TYPE_RECTANGLE, &ret, TRUE, TRUE);')
 
 class ArgMatcher:
     def __init__(self):
@@ -643,6 +676,7 @@ class ArgMatcher:
             self.register('GdkBitmap', oa)
             self.register('GdkBitmap*', oa)
     def register_boxed(self, ptype, typecode):
+        if self.argtypes.has_key(ptype): return
         arg = BoxedArg(ptype, typecode)
         self.register(ptype, arg)
 	self.register(ptype+'*', arg)
@@ -680,7 +714,9 @@ arg = StringArg()
 matcher.register('char*', arg)
 matcher.register('gchar*', arg)
 matcher.register('const-char*', arg)
+matcher.register('char-const*', arg)
 matcher.register('const-gchar*', arg)
+matcher.register('gchar-const*', arg)
 matcher.register('string', arg)
 matcher.register('static_string', arg)
 
@@ -713,7 +749,9 @@ matcher.register('gint8', arg)
 matcher.register('guint16', arg)
 matcher.register('gint16', arg)
 matcher.register('gint32', arg)
-matcher.register('GtkType', arg)
+
+arg = TimeTArg()
+matcher.register('time_t', arg)
 
 # If the system maxint is smaller than unsigned int, we need to use
 # Long objects with PyLong_AsUnsignedLong
@@ -726,6 +764,14 @@ else:
 arg = ULongArg()
 matcher.register('gulong', arg)
 
+arg = Int64Arg()
+matcher.register('gint64', arg)
+matcher.register('long-long', arg)
+
+arg = UInt64Arg()
+matcher.register('guint64', arg)
+matcher.register('unsigned-long-long', arg)
+
 arg = DoubleArg()
 matcher.register('double', arg)
 matcher.register('gdouble', arg)
@@ -745,8 +791,11 @@ matcher.register('GtkType', GTypeArg())
 matcher.register('GError**', GErrorArg())
 matcher.register('GtkTreePath*', GtkTreePathArg())
 matcher.register('GdkRectangle*', GdkRectanglePtrArg())
+matcher.register('GtkAllocation*', GdkRectanglePtrArg())
 matcher.register('GdkRectangle', GdkRectangleArg())
 
+matcher.register('GdkNativeWindow', ULongArg())
+
 matcher.register_object('GObject', None, 'G_TYPE_OBJECT')
 
 del arg
