@@ -15,6 +15,7 @@ class Overrides:
 	self.ignores = {}
 	self.glob_ignores = []
 	self.overrides = {}
+        self.overridden = {}
 	self.kwargs = {}
         self.noargs = {}
         self.startlines = {}
@@ -58,10 +59,10 @@ class Overrides:
 	else:
 	    line = buffer ; rest = ''
 	words = string.split(line)
-	if words[0] == 'ignore':
+	if words[0] == 'ignore' or words[0] == 'ignore-' + sys.platform :
 	    for func in words[1:]: self.ignores[func] = 1
 	    for func in string.split(rest): self.ignores[func] = 1
-	elif words[0] == 'ignore-glob':
+	elif words[0] == 'ignore-glob' or words[0] == 'ignore-glob-' + sys.platform :
 	    for func in words[1:]: self.glob_ignores.append(func)
 	    for func in string.split(rest):
 		self.glob_ignores.append(func)
@@ -104,8 +105,11 @@ class Overrides:
 	return 0
     def is_overriden(self, name):
 	return self.overrides.has_key(name)
+    def is_already_included(self, name):
+        return self.overridden.has_key(name)
     def override(self, name):
-	return self.overrides[name]
+        self.overridden[name] = 1
+        return self.overrides[name]
     def getstartline(self, name):
         return self.startlines[name]
     def wants_kwargs(self, name):
