@@ -224,8 +224,8 @@ static PyTypeObject PyGInterface_Type = {
     0,					/* tp_descr_set */
     0,					/* tp_dictoffset */
     (initproc)pyg_interface_init,	/* tp_init */
-    PyType_GenericAlloc,		/* tp_alloc */
-    PyType_GenericNew,			/* tp_new */
+    (allocfunc)0,			/* tp_alloc */
+    (newfunc)0,				/* tp_new */
     object_free,			/* tp_free */
     (inquiry)0,				/* tp_is_gc */
     (PyObject *)0,			/* tp_bases */
@@ -394,6 +394,7 @@ pyg_object_set_property (GObject *object, guint property_id,
 	PyErr_Print();
 	PyErr_Clear();
     }
+    Py_DECREF(object_wrapper);
     Py_DECREF(py_pspec);
     Py_DECREF(py_value);
 }
@@ -415,8 +416,9 @@ pyg_object_get_property (GObject *object, guint property_id,
 	PyErr_Print();
 	PyErr_Clear();
     }
-    Py_XDECREF(retval);
+    Py_DECREF(object_wrapper);
     Py_DECREF(py_pspec);
+    Py_XDECREF(retval);
 }
 
 static void
@@ -916,7 +918,7 @@ pyg_type_register(PyObject *self, PyObject *args)
 	    return NULL;
 	}
 	PyDict_DelItemString(class->tp_dict, "__gsignals__");
-	Py_DECREF(gsignals);
+	/* Borrowed reference. Py_DECREF(gsignals); */
     } else {
 	PyErr_Clear();
     }
@@ -936,7 +938,7 @@ pyg_type_register(PyObject *self, PyObject *args)
 	    return NULL;
 	}
 	PyDict_DelItemString(class->tp_dict, "__gproperties__");
-	Py_DECREF(gproperties);
+	/* Borrowed reference. Py_DECREF(gproperties); */
     } else {
 	PyErr_Clear();
     }
@@ -1291,6 +1293,7 @@ pyg_error_check(GError **error)
 struct _PyGObject_Functions pygobject_api_functions = {
   pygobject_register_class,
   pygobject_register_wrapper,
+  pygobject_register_sinkfunc,
   pygobject_lookup_class,
   pygobject_new,
 
@@ -1353,12 +1356,16 @@ initgobject(void)
     gerror_exc = PyErr_NewException("gobject.GError", PyExc_RuntimeError,NULL);
     PyDict_SetItemString(d, "GError", gerror_exc);
 
+    PyGObject_Type.tp_alloc = PyType_GenericAlloc;
+    PyGObject_Type.tp_new = PyType_GenericNew;
     pygobject_register_class(d, "GObject", G_TYPE_OBJECT,
 			     &PyGObject_Type, NULL);
     PyDict_SetItemString(PyGObject_Type.tp_dict, "__gdoc__",
 			 pyg_object_descr_doc_get());
 
     PyGInterface_Type.ob_type = &PyType_Type;
+    PyGInterface_Type.tp_alloc = PyType_GenericAlloc;
+    PyGInterface_Type.tp_new = PyType_GenericNew;
     if (PyType_Ready(&PyGInterface_Type))
 	return;
     PyDict_SetItemString(d, "GInterface", (PyObject *)&PyGInterface_Type);
@@ -1371,6 +1378,8 @@ initgobject(void)
 			 pyg_object_descr_doc_get());
 
     PyGBoxed_Type.ob_type = &PyType_Type;
+    PyGBoxed_Type.tp_alloc = PyType_GenericAlloc;
+    PyGBoxed_Type.tp_new = PyType_GenericNew;
     if (PyType_Ready(&PyGBoxed_Type))
 	return;
     PyDict_SetItemString(d, "GBoxed", (PyObject *)&PyGBoxed_Type);
@@ -1379,6 +1388,8 @@ initgobject(void)
     Py_DECREF(o);
 
     PyGPointer_Type.ob_type = &PyType_Type;
+    PyGPointer_Type.tp_alloc = PyType_GenericAlloc;
+    PyGPointer_Type.tp_new = PyType_GenericNew;
     if (PyType_Ready(&PyGPointer_Type))
 	return;
     PyDict_SetItemString(d, "GPointer", (PyObject *)&PyGPointer_Type);
