@@ -28,6 +28,8 @@ void pygtk_input_marshal(gpointer a, PyObject *func, int nargs, GtkArg *args);
 /* private */
 PyObject    *pygtk_tree_path_to_pyobject(GtkTreePath *path);
 GtkTreePath *pygtk_tree_path_from_pyobject(PyObject *object);
+gboolean     pygdk_rectangle_from_pyobject(PyObject *object,
+					   GdkRectangle *rectangle);
 
 GdkAtom pygdk_atom_from_pyobject(PyObject *object);
 
@@ -47,4 +49,8 @@ typedef struct {
 
 PyObject *_pygtk_style_helper_new(GtkStyle *style, int type, gpointer array);
 
+PyObject *_pygtk_tree_model_row_new(GtkTreeModel *model, GtkTreeIter *iter);
+PyObject *_pygtk_tree_model_row_iter_new(GtkTreeModel *model,
+					 GtkTreeIter *parent_iter);
+
 #endif
